# frozen_string_literal: true

module SpiritHub
  class TagsetsController < ApplicationController
    include StateForm

    def logs
      skip_authorization
      skip_policy_scope

      @tagset = Tagset.find(params[:id])
      @logs = @tagset.logs

      respond_to(&:html)
    end

    def doc
      skip_authorization
      skip_policy_scope

      @tagset = Tagset.find(params[:id])
      @logs = @tagset.logs
      @doc = @tagset.docs.find_by(id: params[:doc_id])

      respond_to(&:html)
    end

    def add_doc_form
      skip_authorization
      skip_policy_scope

      tagsets = Tagset.where(type: 'directory').order_by(updated_at: :desc).limit(5)

      render turbo_stream: [
        turbo_stream.update(:modal, partial: 'spirit_hub/tagsets/add_doc_form', locals: { tagsets: })
      ]
    end

    def add_doc
      skip_authorization
      skip_policy_scope

      model = Tagset.create_tagset_doc_from_file(add_doc_params[:files], @user)
      model.add_tagset_doc_to_object(add_doc_params[:tagset_id], @user)
      Log.create_action_log('DOC_CREATED', tagset_id: model.id, user_id: @user.id)

      flash.now[:notice] = 'Document ajouté'
      update_flash_message

      turbo_action_for_add_doc(model)
      add_action(model, add_doc_params)

      render turbo_stream: @turbo_actions
    end

    def share
      skip_authorization
      skip_policy_scope

      model = Tagset.find(params[:id])
      model.share_to_users(**share_params)

      @turbo_actions << turbo_stream.replace("informed_badges_#{model.id}", partial: 'spirit_hub/common/informed_badges', locals: { model: })

      flash.now[:notice] = 'Document partagé'
      update_flash_message

      update_turbo_action_current_state(model)
      add_action(model, share_params)

      render turbo_stream: @turbo_actions
    end

    def classify
      skip_authorization
      skip_policy_scope

      ids = classify_params[:ids].split(',')

      models = Tagset.where(:id.in => ids).to_a

      models.each do |model|
        model.add_tagset_doc_to_object(classify_params[:tagset_id], @user)
      end

      if route_controller_from_referer == 'spirit_hub/directories'
        @turbo_actions = [turbo_stream.reload]
      else
        flash.now[:notice] = 'Documents classés'
        update_flash_message
        classify_turbo_actions(models)
        add_action(models, classify_params)
      end

      render turbo_stream: @turbo_actions
    end

    def validate
      validate_or_refuse(:validate)
    end

    def refuse
      validate_or_refuse(:refuse)
    end

    def search_directories
      skip_authorization
      skip_policy_scope

      tagsets = Tagset.where(
        type: 'directory',
        :operation_id.ne => nil,
        title: /#{Regexp.escape(params[:query]).accent_to_regexp}/i
      ).order_by(updated_at: :desc).limit(5)

      respond_to do |format|
        format.turbo_stream do
          render turbo_stream: [
            turbo_stream.replace(:directory_selector, partial: 'spirit_hub/tagsets/directory_selector', locals: { tagsets: })
          ]
        end
      end
    end

    def add_favorite
      skip_authorization
      skip_policy_scope

      @model = Tagset.find(params[:id])

      if Favorite.create(user: @user, tagset: @model)
        set_user_favorites
        flash.now[:notice] = 'Ajouté aux favoris'

        render turbo_stream: [
          turbo_stream.replace("add_remove_favorites_#{@model.id}", partial: 'spirit_hub/common/add_remove_favorites', locals: { model: @model }),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      else
        flash.now[:alert] = 'Erreur lors de l\'ajout au favoris'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def remove_favorite
      skip_authorization
      skip_policy_scope

      @model = Tagset.find(params[:id])

      if Favorite.where(user: @user, tagset: @model).destroy_all
        set_user_favorites
        flash.now[:notice] = 'Supprimé des favoris'

        render turbo_stream: [
          turbo_stream.replace("add_remove_favorites_#{@model.id}", partial: 'spirit_hub/common/add_remove_favorites', locals: { model: @model }),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      else
        flash.now[:alert] = 'Erreur lors de l\'ajout au favoris'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def delete_item
      skip_authorization
      skip_policy_scope

      @model = Tagset.find(params[:id])

      is_doc = @model.doc?
      tagset_id = @model.id
      title = @model.title

      success_message = is_doc ? 'Document supprimé' : 'Dossier supprimé'

      @model.current_user_id = @user.id

      if @model.destroy
        flash.now[:notice] = success_message
        Log.delete_action_log('DOC_DELETED', user_id: @user.id, tagset_id:, title:) if is_doc

        render turbo_stream: [
          turbo_stream.remove("tagset_#{params[:id]}"),
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
          turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -1 })
        ]
      else
        flash.now[:alert] = 'Erreur lors de la suppresion'

        render turbo_stream: [
          turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages')
        ]
      end
    end

    def bulk_delete
      skip_authorization
      skip_policy_scope

      ids = params[:ids].split(',')

      Tagset.where(:id.in => ids).each do |tagset|
        is_doc = tagset.doc?
        tagset_id = tagset.id
        title = tagset.title

        tagset.current_user_id = @user.id
        tagset.destroy
        Log.delete_action_log('DOC_DELETED', user_id: @user.id, tagset_id:, title:) if is_doc
      end

      flash.now[:notice] = 'Documents supprimés'

      render turbo_stream: [
        *ids.map { |id| turbo_stream.remove("tagset_#{id}") },
        turbo_stream.prepend(:flash_messages, partial: 'spirit_hub/common/flash_messages'),
        turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -ids.length })
      ]
    end

    def form
      skip_authorization
      skip_policy_scope

      models = Tagset.where(:id.in => form_params[:ids].split(',')).to_a

      generate_action_form(form_params[:state_key].to_sym, models)

      render turbo_stream: @turbo_actions
    end

    private

    def share_params
      formatted_params(:id, :subject, :message, :next_state_id, :add_action, :authenticity_token, to: [], cc: [], bcc: [])
    end

    def process_params
      formatted_params(:id, :message, :next_state_id, :add_action, :authenticity_token, recipients: [])
    end

    def classify_params
      formatted_params(:tagset_id, :ids, :next_state_id, :add_action, :authenticity_token)
    end

    def add_doc_params
      formatted_params(:tagset_id, :next_state_id, :add_action, :authenticity_token, files: [])
    end

    def form_params
      formatted_params(:state_key, :ids, :authenticity_token)
    end

    def validate_or_refuse(key)
      skip_authorization
      skip_policy_scope

      model = Tagset.find(process_params[:id])
      model.process_to_users(key, **process_params)

      flash.now[:notice] = key == :refuse ? 'Document refusé' : 'Document validé'
      update_flash_message

      validate_or_refuse_turbo_actions(model)
      add_action(model, process_params)

      render turbo_stream: @turbo_actions
    end

    def validate_or_refuse_turbo_actions(model)
      if ['spirit_hub/doc', 'spirit_hub/tagsets'].include?(route_controller_from_referer)
        @turbo_actions << turbo_stream.remove('doc_validate_refuse')
      else
        update_turbo_action_current_state(model)
      end
    end

    def classify_turbo_actions(models)
      case route_controller_from_referer
      when 'spirit_hub/to_classify'
        models.each do |model|
          @turbo_actions << turbo_stream.remove("tagset_#{model.id}")
        end
        @turbo_actions << turbo_stream.custom_action(:change_counter, target: :end_index, attributes: { amount: -models.length })
      else
        models.each do |model|
          update_turbo_action_current_state(model)
        end
      end
    end

    def turbo_action_for_add_doc(model)
      case route_controller_from_referer
      when 'spirit_hub/favorites'
        @counts = @user.dashboard_counts
        @turbo_actions << turbo_stream.replace('favorites-counts', partial: 'spirit_hub/favorites/counts', formats: [:html])
      when 'spirit_hub/todos'
        handle_todos_add_doc(model)
      when 'spirit_hub/to_classify'
        handle_to_classify_add_doc(model)
      else
        []
      end
    end

    def handle_todos_add_doc(model)
      return [] if model.reload.parent_tagset_ids.blank?

      # TODO : Refactor this
      referer_controller = SpiritHub::TodosController.new
      referer_controller.instance_variable_set(:@user, @user)
      referer_controller.set_columns
      @user_column_names = referer_controller.instance_variable_get(:@user_column_names)
      @user_favorites = []

      partial = referer_controller.set_view_type == 'table' ? 'spirit_hub/todos/row' : 'spirit_hub/todos/card'
      @turbo_actions << turbo_stream.prepend('todos-table-body', partial:, formats: [:html], locals: { model: })
    end

    def handle_to_classify_add_doc(model)
      return [] unless model.reload.parent_tagset_ids.blank?

      # TODO : Refactor this
      referer_controller = SpiritHub::TodosController.new
      referer_controller.instance_variable_set(:@user, @user)
      referer_controller.set_columns
      @user_column_names = referer_controller.instance_variable_get(:@user_column_names)
      @user_favorites = []

      partial = referer_controller.set_view_type == 'table' ? 'spirit_hub/to_classify/row' : 'spirit_hub/to_classify/card'
      @turbo_actions << turbo_stream.prepend('to-classify-table-body', partial:, formats: [:html], locals: { model: })
    end
  end
end
