# frozen_string_literal: true

module SpiritHub
  class DirectoryCompletenessUpdater
    class << self
      def call(*)
        new(*).call
      end
    end

    def initialize(tagset, options = {})
      raise ArgumentError, 'tagset is not a directory' unless tagset&.directory?

      @directory_tagset = tagset.reload
      @options = options
    end

    def call
      if everything_completed?
        mark_directory_as_complete
      else
        mark_directory_as_incomplete
      end

      directory_tagset.save!

      update_parents_completeness if options[:update_parents]

      true
    end

    private

    STATE_COMPLETE = 'Complet'
    STATE_INCOMPLETE = 'À completer'

    attr_reader :directory_tagset, :options

    def everything_completed?
      directory_completed? && children_completed?
    end

    def directory_completed?
      associated_completeness_rules = directory_tagset.completeness.to_a

      associated_completeness_rules.empty? || associated_completeness_rules.all?(&:completed?)
    end

    def children_completed?
      children = directory_tagset.children_tagsets.select(&:directory?).to_a

      children.empty? || children.all?(&:completed?)
    end

    def mark_directory_as_complete
      directory_tagset.content_completeness = 100
      directory_tagset.current_state = STATE_COMPLETE
    end

    def mark_directory_as_incomplete
      directory_tagset.content_completeness = calculate_completeness_percentage
      directory_tagset.current_state = STATE_INCOMPLETE
    end

    def calculate_completeness_percentage
      percentage_sum = 0.0

      # completion percentage from all completeness rules
      associated_completeness_rules = directory_tagset.completeness.to_a
      associated_completeness_rules.each { |record| percentage_sum += 100 if record.completed? }

      # completion percentage from child directories
      children = directory_tagset.children_tagsets.select(&:directory?).to_a
      children.each { |record| percentage_sum += record.content_completeness }

      percentage_sum / (associated_completeness_rules.count + children.count)
    end

    def update_parents_completeness
      # NOTE: All parents should be a directory, but we query for directories only anyway
      directory_tagset.parents.select(&:directory?).each { |parent_tagset| self.class.call(parent_tagset, options) }
    end
  end
end
